/*!
    \file    gd32vw55x_it.c
    \brief   interrupt service routines

    \version 2024-01-19, V1.1.0, demo for GD32VW55x
*/

/*
    Copyright (c) 2024, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

#include "gd32vw55x_it.h"
#include "systick.h"

volatile uint8_t end_flag = 0;

/*!
    \brief      this function handles HardFault exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void HardFault_Handler(void)
{
    /* if Hard Fault exception occurs, go to infinite loop */
    while(1) {
    }
}

/*!
    \brief      this function handles PKCAU exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void PKCAU_IRQHandler(void)
{
    /* address error */
    if(RESET != pkcau_interrupt_flag_get(PKCAU_INT_FLAG_ADDRERR)) {
        pkcau_interrupt_flag_clear(PKCAU_INT_FLAG_ADDRERR);
    }
    /* PKCAU RAM error */
    if(RESET != pkcau_interrupt_flag_get(PKCAU_INT_FLAG_RAMERR)) {
        pkcau_interrupt_flag_clear(PKCAU_INT_FLAG_RAMERR);
    }
    /* end of PKCAU operation */
    if(RESET != pkcau_interrupt_flag_get(PKCAU_INT_FLAG_ENDF)) {
        pkcau_interrupt_flag_clear(PKCAU_INT_FLAG_ENDF);
        end_flag = 1;
    }
}
